#!/usr/bin/perl
# DotDotPwn - The Directory Traversal Fuzzer
# Copyright (C) 2010 Christian Navarrete and Alejandro Hernandez H.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#
# ====================================================================
#
# DotDotPwn - The Directory Traversal Fuzzer
# is a production of:
#
#  CubilFelino                                          Chatsubo
#  Security Research Lab      and       [(in)Security Dark] Labs
#  chr1x.sectester.net                chatsubo-labs.blogspot.com
#
# Changes (Read CHANGELOG.txt for Details):
#
# * DotDotPwn v2.1: The Directory Traversal Fuzzer
#   by chr1x & nitr0us
#
# * DotDotPwn v2.0: Converted into a Fuzzer
#   by nitr0us (nitrousenador@gmail.com)
#   http://chatsubo-labs.blogspot.com
#
# * DotDotPwn v1.0 - Teh Directory Traversal Checker
#   by chr1x@sectester.net
#   http://chr1x.sectester.net
#

use lib qw(.);

$| = 1; # forces a flush after every write or print

## DotDotPwn Core Modules ##
use DotDotPwn::TraversalEngine;

## DotDotPwn Protocol Modules ##
use DotDotPwn::HTTP;
use DotDotPwn::HTTP_Url;
use DotDotPwn::FTP;
use DotDotPwn::TFTP;
use DotDotPwn::Payload;
use DotDotPwn::STDOUT;

## DotDotPwn Misc Modules ##
use DotDotPwn::Fingerprint;

## Perl modules ##
use Getopt::Std;
use Switch;

my $DotDotPwn  =
'#################################################################################
#                                                                               #
#  CubilFelino                                                       Chatsubo   #
#  Security Research Lab              and            [(in)Security Dark] Labs   #
#  chr1x.sectester.net                             chatsubo-labs.blogspot.com   #
#                                                                               #
#                               pr0udly present:                                #
#                                                                               #
#  ________            __  ________            __  __________                   #
#  \______ \    ____ _/  |_\______ \    ____ _/  |_\______   \__  _  __ ____    #
#   |    |  \  /  _ \\\\   __\|    |  \  /  _ \\\\   __\|     ___/\ \/ \/ //    \   #
#   |    `   \(  <_> )|  |  |    `   \(  <_> )|  |  |    |     \     /|   |  \  #
#  /_______  / \____/ |__| /_______  / \____/ |__|  |____|      \/\_/ |___|  /  #
#          \/                      \/                                      \/   #
#                               - DotDotPwn v2.1 -                              #
#                         The Directory Traversal Fuzzer                        #
#                         http://dotdotpwn.sectester.net                        #
#                            dotdotpwn@sectester.net                            #
#                                                                               #
#                              by chr1x & nitr0us                               #
#################################################################################

';

if(@ARGV < 2){ # -m module required
	print $DotDotPwn; # Banner

	print "Usage: $0 -m <module> -h <host> [OPTIONS]\n";
        print "\tAvailable options:\n";
	print "\t-m\tModule [http | http-url | ftp | tftp | payload | stdout]\n";
	print "\t-h\tHostname\n";
	print "\t-O\tOperating System detection for intelligent fuzzing (nmap)\n";
	print "\t-s\tService version detection (banner grabber)\n";
	print "\t-d\tDeep of traversals (e.g. deep 3 equals to ../../../; default: 6)\n";
	print "\t-f\tSpecific filename (e.g. /etc/motd; default: filenames defined in the TraversalEngine.pm)\n";
	print "\t-u\tURL with the part to be fuzzed marked as TRAVERSAL (e.g. http://foo:8080/id.php?x=TRAVERSAL&y=31337)\n";
	print "\t-k\tString pattern to match in the response if it's vulnerable (e.g. \"root:\" if trying with /etc/passwd)\n";
	print "\t-U\tUsername (default: 'anonymous')\n";
	print "\t-P\tPassword (default: 'dot\@dot.pwn')\n";
	print "\t-p\tFilename with the payload to be sent and the part to be fuzzed marked as TRAVERSAL\n";
	print "\t-x\tPort to connect (default: HTTP=80; FTP=21; TFTP=69)\n";
	print "\t-t\tTime in milliseconds between each test (default: 300 (.3 second))\n";
	print "\t-b\tBreak after the first vulnerability is found\n";
	print "\t-q\tQuiet mode (doesn't print each attemp)\n";

	exit;
}

getopts("qOsbm:h:U:P:f:u:k:d:x:t:p:");

our $module  = $opt_m || die "Module is neccesary (-m)\n";
my $host     = $opt_h || die "Hostname is neccesary (-h)\n" unless ($module eq "http-url" || $module eq "stdout");
my $user     = $opt_U || 'anonymous';
my $pass     = $opt_P || 'dot@dot.pwn';
our $quiet   = $opt_q;
our $break   = $opt_b;
my $url      = $opt_u;
our $pattern = $opt_k;
my $file     = $opt_f;
my $deep     = $opt_d || 6;
my $OS       = $opt_O;
my $serv     = $opt_s;
our $time    = ($opt_t || 300) * 1000; # Time in milliseconds between each test
our $start_time; # Will hold the time at the beginning of execution
our $runtime;   # Will hold the difference betweend the end time and $start_time, so, it's the runtime
my $payload_file = $opt_p;
my $payload; # The content of $payload_file
my $proto_url;
my $proto;
my $port;
our $dot_quiet_mode = 10; # When quiet mode is enabled, print a dot (.) each 10 attempts
my $n_travs  = 0; # Counter of Traversals found
our @traversals; # Traversal strings generated by the Traversal Engine that will be launched against the target

print $DotDotPwn if $module ne "stdout";


# Variable asignment and other validations per module
switch($module){
	case "ftp"  {	$port = $opt_x || 21;	}
	case "http" {	$port = $opt_x || 80;	}
	case "tftp" {	$port = $opt_x || 69;	}
	case "http-url" {
		die "URL is neccesary (-u)\n" unless $url;
		die "Pattern string to match is neccesary (-k)\n" unless $pattern;

		# URL Parsing
		die "Invalid URL format!\n" if $url !~ m|(\w+)://([\w\.]+):?(\d*)?/|;

		$proto_url = $1;
		$host  = $2;
		$port = $3 || 80;

		die "'$proto_url' Protocol not supported\n" if $proto_url ne "http";

		die "No \"TRAVERSAL\" keyword found in the supplied URL\n" if $url !~ /TRAVERSAL/;
	}
	case "payload" {
		$port = $opt_x || die "Port number is necessary (-x)\n";
		die "Payload file is necessary (-p)\n" unless $payload_file;
		die "Pattern string to match is neccesary (-k)\n" unless $pattern;

		open PAYLOAD_FD, $payload_file or die "Cannot open $payload_file: $!";

		# Undef the end of record character to read the whole file into one scalar variable
		undef $/;

		$payload = <PAYLOAD_FD>;

		close PAYLOAD_FD;

		$/ = "\n"; # Restore for normal behaviour

		die "No \"TRAVERSAL\" keyword found in the supplied payload file\n" if $payload !~ /TRAVERSAL/;
	}
	case "stdout" {
		@traversals = TraversalEngine("generic", $deep, $file);
		toSTDOUT();
		exit;
	}
	else { print "[-] Invalid Module ($module)!\n"; exit; }
}

# Target information
print "\n[========== TARGET INFORMATION ==========]\n";
print "[+] Hostname: $host\n";

if($OS){
	print "[+] Detecting Operating System (nmap) ...\n";
	$target_OS = OS_Detection($host);
	print "[+] Operating System detected: " . $target_OS . "\n";
}

$proto = $proto_url || ($module eq "payload" ? "N/A" : $module);

print "[+] Protocol: $proto\n";
print "[+] Port: $port\n";
print "[+] Service detected:\n" . Banner_Grabber($host, $port, $proto) if $serv;


# Traversal Engine
print "\n[=========== TRAVERSAL ENGINE ===========]\n";
@traversals = TraversalEngine(OS_type($target_OS), $deep, $file);
print "[+] Traversal Engine DONE ! - Total traversal tests created: " . scalar(@traversals) . "\n";


# Testing
print "\n[=========== TESTING RESULTS ============]\n";
printf "[+] Ready to launch %.2f traversals per second\n", (1000000 / $time);
print "[+] Press any key to start the testing (You can stop it pressing Ctrl + C)\n";
<STDIN>;

$start_time = time;

# (nitr0us)
# "use Switch" Added here again to avoid an existing bug in Switch.pm @ Perl 5.8 
# that raises an error in the next switch($module) statement
#
# http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=480106
use Switch;

switch($module){
	case "ftp"	{ $n_travs = FuzzFTP($host, $port, $user, $pass); }
	case "http"	{ $n_travs = FuzzHTTP($host, $port); }
	case "tftp"	{ $n_travs = FuzzTFTP($host, $port); }
	case "payload"	{ $n_travs = FuzzPayload($host, $port, $payload); }
	case "http-url"	{ $n_travs = FuzzHTTP_Url($url); }
}

$runtime = time - $start_time;
printf "\n[+] Fuzz testing finished after %.2f minutes ($runtime seconds)\n", ($runtime / 60);
print "[+] Total Traversals found: $n_travs\n";

exit;
